package com.plugin.techprocess.techprocess

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Intent
import android.util.Log
import android.widget.Toast
import androidx.annotation.NonNull
import androidx.core.app.ActivityCompat.startActivityForResult
import com.paynimo.android.payment.PaymentActivity
import com.paynimo.android.payment.PaymentModesActivity
import com.paynimo.android.payment.model.Checkout
import com.paynimo.android.payment.util.Constant
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import java.text.SimpleDateFormat
import java.util.*


/** TechprocessPlugin */
class TechprocessPlugin : FlutterPlugin, PluginRegistry.ActivityResultListener, MethodCallHandler,
    ActivityAware {

    private lateinit var channel: MethodChannel
    private lateinit var result: Result
    private var activity: Activity? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "techprocess")
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "getPlatformVersion") {
            result.success("Android ${android.os.Build.VERSION.RELEASE}")
        } else if (call.method == "callTechProcess") {
            startTransaction(call)
            this.result = result
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    private fun startTransaction(call: MethodCall) {
        val arg = call.arguments as Map<*, *>?
        if (arg != null) {
            val techMerchantCode = arg["techMerchantCode"] as String?
            val transactionNo = arg["transactionNo"] as String?
            val techTransactionReference = arg["techTransactionReference"] as String?
            val techTransactionType = arg["techTransactionType"] as String?
            val techTransactionSubType = arg["techTransactionSubType"] as String?
            val techCurrency = arg["techCurrency"] as String?
            val amount = arg["amount"] as String?
            val emailId = arg["emailId"] as String?
            val mobileNo = arg["mobileNo"] as String?
            val schemaCode = arg["schemaCode"] as String?
            val publicKey = arg["publicKey"] as String?
            val mode = arg["mode"] as String?


            try {
                val checkout = Checkout();
                checkout.setMerchantIdentifier(techMerchantCode);
                checkout.setTransactionIdentifier(transactionNo);
                checkout.setTransactionReference(techTransactionReference);
                checkout.setTransactionType(techTransactionType);
                checkout.setTransactionSubType(techTransactionSubType);
                checkout.setTransactionCurrency(techCurrency);
                checkout.setTransactionAmount(amount);
                checkout.setTransactionDateTime(getStatementDate());

                // Setting values Consumer fields
                checkout.setConsumerIdentifier("");
                checkout.setConsumerEmailID(emailId);
                checkout.setConsumerMobileNumber(mobileNo);
                checkout.setConsumerAccountNo("");

                checkout.addCartItem(
                    schemaCode,
                    amount,
                    "0.0",
                    "0.0",
                    "",
                    "",
                    "",
                    "www.shriramcity.in"
                );

                Log.i("Checkout Request Object", checkout.getMerchantRequestPayload().toString());

                val authIntent =
                    PaymentModesActivity.Factory.getAuthorizationIntent(activity, true);
                authIntent.putExtra(Constant.ARGUMENT_DATA_CHECKOUT, checkout);
                authIntent.putExtra(PaymentActivity.EXTRA_PUBLIC_KEY, publicKey);

                when (mode) {
                    "UPI" -> {
                        authIntent.putExtra(
                            PaymentActivity.EXTRA_REQUESTED_PAYMENT_MODE,
                            PaymentActivity.PAYMENT_METHOD_UPI
                        );
                    }
                    "WL" -> {
                        authIntent.putExtra(
                            PaymentActivity.EXTRA_REQUESTED_PAYMENT_MODE,
                            PaymentActivity.PAYMENT_METHOD_WALLETS
                        );
                    }
                    "NB" -> {
                        authIntent.putExtra(
                            PaymentActivity.EXTRA_REQUESTED_PAYMENT_MODE,
                            PaymentActivity.PAYMENT_METHOD_NETBANKING
                        );
                    }
                    "DC" -> {
                        authIntent.putExtra(
                            PaymentActivity.EXTRA_REQUESTED_PAYMENT_MODE,
                            PaymentActivity.PAYMENT_METHOD_CARDS
                        );
                    }
                }
                val settings = PaymentModesActivity.Settings();
                authIntent.putExtra(Constant.ARGUMENT_DATA_SETTING, settings);

                startActivityForResult(activity!!, authIntent, PaymentActivity.REQUEST_CODE, null);


            } catch (e: Exception) {
                Log.e("callTechProcessExp", e.toString());
                showToast("callTechProcessExp "+ e.toString())

            }

        } else {
            showToast("Please send arguments")

        }
    }


    private fun setResult(message: String?, value: HashMap<String, String?>? = null) {
        result.error("0", message ?: "Unknown error", value)
    }

    private fun setResult(value: HashMap<String, String?>) {
        result.success(value)
    }

    private fun showToast(message: String) {
        Toast.makeText(activity!!, message, Toast.LENGTH_LONG).show()
    }

    override fun onDetachedFromActivity() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
        binding.addActivityResultListener(this)
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }

    @SuppressLint("SimpleDateFormat")
    fun getStatementDate(): String? {
        val format = SimpleDateFormat("dd-MM-yyy")
        val date: java.util.Date = Date()
        return format.format(date)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        if (!::result.isInitialized) {
            return false
        }
        try {
            if (resultCode == PaymentActivity.RESULT_OK && data != null) {
                val checkout_res =
                    data.getSerializableExtra(Constant.ARGUMENT_DATA_CHECKOUT) as Checkout
                Log.i("CheckoutResponseObject", checkout_res.merchantResponsePayload.toString())
                val transactionSubType = checkout_res.merchantRequestPayload.transaction.subType
                Log.i("Payment type => ", transactionSubType)
                if (checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.statusCode.equals(
                        PaymentActivity.TRANSACTION_STATUS_SALES_DEBIT_SUCCESS,
                        ignoreCase = true
                    )
                ) {
                    Log.i("TRANSACTION STATUS=>", "SUCCESS")
                    if (checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.instruction.id != null && checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.instruction.id.isEmpty()) {
                        Log.i("TRANSACTION SI STATUS=>", "SI Transaction Not Initiated")
                    } else if (checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.instruction.id != null && !checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.instruction.id.isEmpty()) {
                        Log.i("TRANSACTION SI STATUS=>", "SUCCESS")
                    }
                } else if (checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.statusCode.equals(
                        PaymentActivity.TRANSACTION_STATUS_DIGITAL_MANDATE_SUCCESS,
                        ignoreCase = true
                    )
                ) {
                    Log.i("TRANSACTION STATUS=>", "SUCCESS")
                    if (checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.instruction.id != null && !checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.instruction.id.isEmpty()) {
                        Log.i("TRANSACTION SI STATUS=>", "INITIATED")
                    } else {
                        Log.i("TRANSACTION SI STATUS=>", "FAILURE")
                    }
                } else {
                    // some error from bank side
                    Log.i("TRANSACTION STATUS=>", "FAILURE")
                }
                val statusCode =
                    checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.statusCode
                val statusMessage =
                    checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.statusMessage
                val identifier =
                    checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.identifier
                val amount =
                    checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.amount
                val result = """
            
            StatusCode : $statusCode
            StatusMessage : $statusMessage
            ErrorMessage : ${checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.errorMessage}
            Amount : $amount
            DateTime : ${checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.dateTime}
            MerchantTransactionIdentifier : ${checkout_res.merchantResponsePayload.merchantTransactionIdentifier}
            Identifier : $identifier
            BankSelectionCode : ${checkout_res.merchantResponsePayload.paymentMethod.bankSelectionCode}
            BankReferenceIdentifier : ${checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.bankReferenceIdentifier}
            RefundIdentifier : ${checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.refundIdentifier}
            BalanceAmount : ${checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.balanceAmount}
            InstrumentAliasName : ${checkout_res.merchantResponsePayload.paymentMethod.instrumentAliasName}
            SI Mandate Id : ${checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.instruction.id}
            SI Mandate Status : ${checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.instruction.statusCode}
            SI Mandate Error Code : ${checkout_res.merchantResponsePayload.paymentMethod.paymentTransaction.instruction.errorcode}
            """.trimIndent()
                Log.i("TechProcessResponse", result)
                val res = HashMap<String, String?>()
                res["statusCode"]= statusCode
                res["statusMessage"]= statusMessage
                res["response"]= checkout_res.merchantResponsePayload.toString()
                setResult(res);
                return true;
            } else if (data != null) {
                if (resultCode == PaymentActivity.RESULT_ERROR && data.hasExtra(PaymentActivity.RETURN_ERROR_CODE) && data.hasExtra(
                        PaymentActivity.RETURN_ERROR_DESCRIPTION
                    )
                ) {
                    if (data != null) {
                        Log.e(
                            "TechProcessResponse",
                            data.getStringExtra(PaymentActivity.RETURN_ERROR_CODE)
                                .toString() + "|" + data.getStringExtra(PaymentActivity.RETURN_ERROR_DESCRIPTION)
                        )
                    }
                    if (data != null) {
                        val res = HashMap<String, String?>()
                        res["errorCode"]= data.getStringExtra(PaymentActivity.RETURN_ERROR_CODE)
                        res["errorMessage"]= data.getStringExtra(PaymentActivity.RETURN_ERROR_DESCRIPTION)
                        setResult(res);
                        return true;
                    }
                } else if (resultCode == PaymentActivity.RESULT_CANCELED) {
                    Log.e("TechProcessResponse", "Back pressed, Transaction Cancelled")
                    val res = HashMap<String, String?>()
                    res["errorMessage"]= "Back pressed, Transaction Cancelled"
                    setResult(res);
                    return true;
                } else {
                    val res = HashMap<String, String?>()
                    res["errorMessage"]= "Transaction Cancelled"
                    setResult(res);
                    return true;
                }
            }
        } catch (e: Exception) {
            Log.e("TechProcessException", e.toString())
            val res = HashMap<String, String?>()
            res["errorMessage"]= "TechProcessException"
            setResult(res);
            return true;
        }
        return false
    }
}
